function makeInfo = rtwmakecfg

makeInfo = [];

plecsRSimTarget = false;
plecsRealTimeTarget = false;

% check what Simulink is building for
rtwGenSettings = get_param(bdroot, 'RTWGenSettings');
if strcmp(rtwGenSettings.CodeFormat, 'Accelerator_S-Function'),
  % exit here for simulations in Accelerator mode
  % Simulink will just create a wrapper around the plecs mex function
  return
end

rtwIsRSimTarget = (isfield(rtwGenSettings, 'IsRSim') ...
                   && strcmpi(rtwGenSettings.IsRSim, 'yes')) ...
                  || strcmp(rtwGenSettings.CodeFormat, 'S-Function');

% check the individual target settings of the circuit blocks
circuits = find_system(bdroot, ...
  'FollowLinks', 'on', ...
  'LookUnderMasks', 'all', ...
  'MaskType', 'PlecsCircuit');
for circuit = circuits',
  cctTarget = plecs('get', char(circuit), 'RTWTarget');
  if strcmp(cctTarget,'RSim') ...
     || (~strcmp(cctTarget,'RealTime') && rtwIsRSimTarget),
    plecsRSimTarget = true;
  else
    plecsRealTimeTarget = true;
  end
end

if plecsRSimTarget,
  [libDir, dllDir, libName] = iRSimDetermineLibDirAndName;
  if strncmpi(get_param(bdroot, 'SimulationMode'), 'rapid', 5) ...
     || strcmp(rtwGenSettings.CodeFormat, 'S-Function'),
    % if this is a Rapid Accelerator or an S-Function build, make sure that
    % the appropriate environment variables are set so that MATLAB will find
    % the plecsrsim library and plecsrsim will find the license file
    iRSimSetEnv(dllDir);
  end
  makeInfo.linkLibsObjs = { rtw_alt_pathname(fullfile(libDir, libName)) };
end
if plecsRealTimeTarget,
  plecsRoot = fileparts(which('plecs'));
  srcDir = rtw_alt_pathname(fullfile(plecsRoot, 'rtw', 'src', ''));
  makeInfo.includePath = {srcDir};
  makeInfo.sourcePath = {srcDir};
  makeInfo.sources = {'plecsrtw.c'};
end
% end function

%
% determine the architecture and the name of the dynamic library
%
function [libDir, dllDir, libName] = iRSimDetermineLibDirAndName

plecsRoot = fileparts(which('plecs'));
binDir = fullfile(plecsRoot, 'bin', '');
switch mexext,
  case {'mexw32', 'dll'}
    libName = 'plecsrsim.lib';
    dllDir = fullfile(binDir, 'win32', '');
    compiler = plcheckmex;
    if strcmp(compiler, 'lcc'),
      libDir = fullfile(dllDir, 'lcc', '');
    else
      if ~strcmp(compiler, 'cl'),
        warning('Unknown compiler ''%s''. Using MSVC import library for plecsrsim.dll.', compiler); %#ok<WNTAG>
      end
      libDir = fullfile(dllDir, 'msvc', '');
    end
  case 'mexw64',
    libName = 'plecsrsim.lib';
    dllDir = fullfile(binDir, 'win64', '');
    libDir = dllDir;
  case 'mexmaci',
    libName = 'libplecsrsim.dylib';
    dllDir = fullfile(binDir, 'maci', '');
    libDir = dllDir;
  case 'mexmaci64',
    libName = 'libplecsrsim.dylib';
    dllDir = fullfile(binDir, 'maci64', '');
    libDir = dllDir;
  case 'mexglx',
    libName = 'libplecsrsim.so';
    dllDir = fullfile(binDir, 'glnx86', '');
    libDir = dllDir;
  case 'mexa64',
    libName = 'libplecsrsim.so';
    dllDir = fullfile(binDir, 'glnxa64', '');
    libDir = dllDir;
  otherwise,
    error('Unsupported architecture for RSim target');
end
% end function

%
% set the search path for loading the dynamic library and for FLEXnet
% (if they haven't been set yet)
%
function iRSimSetEnv(dllDir)

if strncmpi(computer, 'pc', 2),
  envVar = 'PATH';
elseif strncmpi(computer, 'mac', 3),
  envVar = 'DYLD_LIBRARY_PATH';
else
  envVar = 'LD_LIBRARY_PATH';
end
p = getenv(envVar);
if ~any(strfind(p, dllDir)),
  setenv(envVar, [p pathsep dllDir]);
end

if isempty(getenv('PLEXIM_LICENSE_FILE')),
  plecsRoot = fileparts(which('plecs'));
  setenv('PLEXIM_LICENSE_FILE', fullfile(plecsRoot, 'license.dat'));
end
% end function

